#import "MyView.h"
#import "sprite.h"

#define normal 0
#define movingSprite 1
#define drawingSquare 2
#define drawingBubble 3
#define drawingThoughtBubble 4
#define makingTail1 5
#define makingTail2 6
#define makingTail3 7
#define drawingDialog 8
#define drawingTinyBubbles 9
#define doingCrop 10
#define performingCrop 11

@implementation MyView

- (id)initWithFrame:(NSRect)frameRect
{
    self = [super initWithFrame:frameRect];
    
    [NSBezierPath setDefaultLineWidth: 2];
    
    background = [[NSImage alloc] initWithSize: NSMakeSize(frameRect.size.width, frameRect.size.height)];
    handC = [[NSCursor alloc] initWithImage:[[NSImage allocWithZone:[self zone]] initWithContentsOfFile: [[NSBundle bundleForClass:[self class]] pathForResource:@"handCursor" ofType:@"tiff"]] hotSpot:NSMakePoint(8, 8)]; // init "thing"
    boxC = [[NSCursor alloc] initWithImage:[[NSImage allocWithZone:[self zone]] initWithContentsOfFile: [[NSBundle bundleForClass:[self class]] pathForResource:@"squareCursor" ofType:@"tiff"]] hotSpot:NSMakePoint(8, 8)]; 
    bubbleC = [[NSCursor alloc] initWithImage:[[NSImage allocWithZone:[self zone]] initWithContentsOfFile: [[NSBundle bundleForClass:[self class]] pathForResource:@"bubbleCursor" ofType:@"tiff"]] hotSpot:NSMakePoint(13, 15)];
    thoughBubbleC = [[NSCursor alloc] initWithImage:[[NSImage allocWithZone:[self zone]] initWithContentsOfFile: [[NSBundle bundleForClass:[self class]] pathForResource:@"thoughtBubbleCursor" ofType:@"tiff"]] hotSpot:NSMakePoint(13, 15)];
    cropC = [[NSCursor alloc] initWithImage:[[NSImage allocWithZone:[self zone]] initWithContentsOfFile: [[NSBundle bundleForClass:[self class]] pathForResource:@"cropCursor" ofType:@"tiff"]] hotSpot:NSMakePoint(7, 4)];

    spriteThatsMoving = [[NSImage alloc] initWithSize: NSMakeSize (32, 72)];
    bubble = [[NSImage alloc] initWithSize: NSMakeSize (32, 72)];
    tailOutline = [NSBezierPath bezierPath];
    trackingRects = [NSMutableArray arrayWithCapacity :1];
    dialogToRender = [NSMutableString stringWithCapacity:10];
    [dialogToRender retain];
    [handC retain];
    [boxC retain];
    [bubbleC retain];
    [thoughBubbleC retain];
    [allSprites retain];
    [trackingRects retain];
	[dialogTracking retain];
    [bubble retain];
    [tailOutline retain];
    layerWorkingOn = 0;
    state = normal;
    lastRectTouched = -1;
    lastSpriteTouched = -1;
    rectTouchedCounter = 0;
    return self;
   }

- (void)useTheseThings :(NSMutableArray *)theseLayers :(NSMutableArray *)theseSprites :(NSMutableArray *)borderArray :(NSMutableArray *)dialogArray
{
    layers = theseLayers; // I'm keeping around a pointer to the array in the controller.
    [layers retain];
    allSprites = theseSprites;
    [allSprites retain];
    bordersArray = [borderArray retain];
	dialogTracking = [dialogArray retain];
}

- (void)addCharacter: (NSImage *)spriteToAdd :(int)layer :(int)xCord :(int)yCord
{
    sprite *newSprite = [[sprite alloc] init: spriteToAdd :NSMakeRect(0, 0, [spriteToAdd size].width, [spriteToAdd size].height) :layer];
    [newSprite setRect :NSMakeRect (xCord, yCord, [spriteToAdd size].width, [spriteToAdd size].height)];
    [allSprites addObject :newSprite];

}

- (BOOL)acceptsFirstResponder
{
    return YES;
}

- (BOOL)becomeFirstResponder
{
    [self setNeedsDisplay:YES];
    return YES;
}

- (BOOL)shouldEdit
{
	int lcv;
	for ( lcv = 0; lcv < [dialogTracking count]; lcv++ ){
		if ( [[[dialogTracking objectAtIndex: lcv] objectForKey:@"spriteNumber"] intValue] == lastSpriteTouched )
		return TRUE;
	}
	return FALSE;
}


- (void)changeLayer :(int)layerToChangeTo
{
    int lcv;
    for ( lcv = 0; lcv < [layers count]; lcv++ ){
        if ( [[[layers objectAtIndex :lcv ] objectForKey :@"layer"] intValue ] == layerToChangeTo ){
            layerWorkingOn = lcv;
        }
    }
    lastRectTouched = -1;
    lastSpriteTouched = -1;
    [self setNeedsDisplay :YES];
}

- (void)copySprite
{
    if ( lastSpriteTouched != -1 ){
        spriteClipboard = [[allSprites objectAtIndex :lastSpriteTouched] retain];
        [self setNeedsDisplay:YES];
        lastSpriteTouched = -1;
    }    
}

- (void)cutSprite
{
	spriteClipboard = [[allSprites objectAtIndex :lastSpriteTouched] retain];
	[self deleteSprite];
	
}
- (void)deleteSprite
{
	int lcv;
	
	for ( lcv = 0; lcv < [dialogTracking count]; lcv++ )
		if ( [[[dialogTracking objectAtIndex: lcv] objectForKey:@"spriteNumber"] intValue] == lastSpriteTouched )
			[dialogTracking removeObjectAtIndex: lcv];
	
	for ( lcv = 0; lcv < [dialogTracking count]; lcv++ ){ // I need to reduce the spriteNumber of anything after the one I've just deleted
		if ( [[[dialogTracking objectAtIndex: lcv] objectForKey:@"spriteNumber"] intValue] >= lastSpriteTouched ){
			[[dialogTracking objectAtIndex: lcv] setValue:[NSString stringWithFormat:@"%d", [[[dialogTracking objectAtIndex: lcv] objectForKey:@"spriteNumber"] intValue] - 1] forKey:@"spriteNumber"];
		}
	}

	NSLog ([dialogTracking description]);
	
    if ( lastSpriteTouched != -1 ){
        [allSprites removeObjectAtIndex :lastSpriteTouched];
        [self setNeedsDisplay:YES];
        lastSpriteTouched = -1;
    }
}

- (void)drawRect:(NSRect)rect
{
    int lcv, lcv2, borderLCV;
//    NSLog (@"%@", [layers description]);
    for ( lcv = 0; lcv < [trackingRects count]; lcv++ )
        [self removeTrackingRect:[[trackingRects objectAtIndex:lcv] intValue]];
//    NSLog ([layers description]);
    [trackingRects removeAllObjects];
    [[NSColor whiteColor] set];
    NSRectFill(rect);
    [[NSColor blackColor] set];
    for (lcv = 1; lcv < [layers count] + 1; lcv++ ){
        if ( lcv == 2 ){
            [[NSColor blackColor] set];
            for ( borderLCV = 0; borderLCV < [bordersArray count]; borderLCV++ )
                [[NSBezierPath bezierPathWithRect:
                    NSMakeRect([[[bordersArray objectAtIndex :borderLCV] objectForKey :@"startX"] floatValue], [[[bordersArray objectAtIndex :borderLCV] objectForKey :@"startY"] floatValue], [[[bordersArray objectAtIndex :borderLCV] objectForKey :@"differenceX"] intValue], [[[bordersArray objectAtIndex :borderLCV] objectForKey :@"differenceY"] intValue])]
                    stroke];
        }            
//        NSLog (@"Now checking layer %@", [[layers objectAtIndex :lcv - 1] objectForKey :@"layer"] );
        for ( lcv2 = 0; lcv2 < [allSprites count]; lcv2++ ){
 //           NSLog (@"THis spirte is in layer %d", [[allSprites objectAtIndex: lcv2] layer] );
            if ( [[allSprites objectAtIndex: lcv2] layer] == [[[layers objectAtIndex :lcv - 1] objectForKey :@"layer"] intValue] ){ // build up each layer
                if ( lcv2 == lastSpriteTouched ){
                    [[NSColor colorWithCalibratedRed:1.0 green:0.0 blue:0.0 alpha:.2] set];
                    [[NSBezierPath bezierPathWithRect:[[allSprites objectAtIndex: lcv2] currentRect] ] fill];
                }
                [[[allSprites objectAtIndex: lcv2] sprite] dissolveToPoint:NSMakePoint([[allSprites objectAtIndex: lcv2] currentRect].origin.x, [[allSprites objectAtIndex: lcv2] currentRect].origin.y)
                        fromRect:NSMakeRect(0, 0, [[[allSprites objectAtIndex: lcv2] sprite] size].width, [[[allSprites objectAtIndex: lcv2] sprite] size].height)
                        fraction:[[allSprites objectAtIndex: lcv2] transparency] * .01];
                if ( layerWorkingOn == lcv - 1 ){
                    [[NSColor blueColor] set];
                    [[NSBezierPath bezierPathWithRect:[[allSprites objectAtIndex: lcv2] currentRect] ] stroke];
                    [trackingRects addObject :[NSString stringWithFormat :@"%d", [self addTrackingRect: [[allSprites objectAtIndex: lcv2] currentRect] owner:self userData:[[NSString alloc] initWithFormat :@"%d", lcv2] assumeInside:NO]]];
                }
            }
        }
    }

}

- (void)deleteLayer:(int)layerToDelete
{
    int lcv;
    for ( lcv = 0; lcv < [allSprites count]; lcv++ ){
        if ( [[allSprites objectAtIndex: lcv] layer] == layerToDelete  ){ // check each
//            NSLog (@"Deleting a sprite from layer %d", layerToDelete);
            [allSprites removeObjectAtIndex: lcv];
            lcv--;
        }
    }
    lastRectTouched = -1;
    lastSpriteTouched = -1;
}

- (void)mirrorSprite :(int)X :(int)Y
{
    if ( lastSpriteTouched != -1 ){
        NSImage *tempImage = [[NSImage alloc] initWithSize: [[[allSprites objectAtIndex :lastSpriteTouched] sprite] size]];
        // tempImage is for the catch-22: I have to clear the old sprite out, but I also have to draw it in, at the same time!
        NSAffineTransform* transform = [NSAffineTransform transform];
        [transform scaleXBy:X yBy:Y];
        if ( X == -1 )
            [transform translateXBy :-[tempImage size].width yBy:0]; // I have to translate it back b/c scaleX -1 mirrors, so now
        if ( Y == -1 )
            [transform translateXBy :0 yBy:-[tempImage size].height];
        // the image is at -[size], this puts it back
// first, get the flipped graphic into the temp
        [tempImage lockFocus];
        [transform concat];
        [[[allSprites objectAtIndex :lastSpriteTouched] sprite] drawAtPoint:NSMakePoint(0, 0 ) fromRect:NSMakeRect(0,0, [[[allSprites objectAtIndex :lastSpriteTouched] sprite] size].width, [[[allSprites objectAtIndex :lastSpriteTouched] sprite] size].height ) operation:NSCompositeSourceOver fraction:1];
        [tempImage unlockFocus];
        [[NSAffineTransform transform] set];
// now, clear out the old, and put the temp in it's place
        [[[allSprites objectAtIndex :lastSpriteTouched] sprite ] lockFocus];
        [[NSColor clearColor] set];
        NSRectFill (NSMakeRect(0,0, [[[allSprites objectAtIndex :lastSpriteTouched] sprite] size].width, [[[allSprites objectAtIndex :lastSpriteTouched] sprite] size].height ));
        [tempImage drawAtPoint :NSMakePoint(0, 0 ) fromRect:NSMakeRect(0,0, [tempImage size].width, [tempImage size].height ) operation:NSCompositeSourceOver fraction:1];
        [[[allSprites objectAtIndex :lastSpriteTouched] sprite] unlockFocus];
        [self setNeedsDisplay:YES];
    }    
}

- (void)mouseEntered:(NSEvent *)theEvent
{
    if ( state == performingCrop )
        return; // don't bother me now, fool!
   if ( state != movingSprite )
       rectTouchedCounter++;
//    NSLog (@"%d", rectTouchedCounter);
    if ( state == normal || state == doingCrop){
        lastRectTouched = [[theEvent userData] intValue]; // see, right here. That's the number of the rect
    // come on, it's an int, stop complaining about it! (compile, you'll see)
    }
}

- (void)mouseExited:(NSEvent *)event
{

    if ( state != movingSprite )
        rectTouchedCounter--;
    if ( rectTouchedCounter < 0 )
        rectTouchedCounter = 0;
//    NSLog (@"%d", rectTouchedCounter);
    // I'm not sure what this was used for, but it seems to be work ok without it, so until something
    // goes wrong and I have to put it back, or I'm satisfied it does nothing, let's leave it like this
//    if ( rectTouchedCounter > 1 )
//        return;
    if ( state == normal ){ // !!!!! Darn it, I still have this problem. I get a mouseEntered and then an exited when I go
        // from one rect to another.
        lastRectTouched = -1;
    }
}

- (void)mouseDown:(NSEvent *)theEvent
{
    int lcv, lcv2;

    if ( lastRectTouched != -1 && state == normal){
        lastSpriteTouched = lastRectTouched;
NSLog (@"%d", lastSpriteTouched);
        [handC set];
        [spriteThatsMoving setSize: [[[allSprites objectAtIndex :lastRectTouched] sprite] size]];
        [spriteThatsMoving lockFocus];
        [[NSColor clearColor] set];
        NSRectFill(NSMakeRect(0,0, [spriteThatsMoving size].width, [spriteThatsMoving size].height));
        [[[allSprites objectAtIndex :lastRectTouched] sprite] compositeToPoint:NSMakePoint(0, 0)
                                        fromRect:NSMakeRect(0,0, [[allSprites objectAtIndex :lastRectTouched] currentRect].size.width, [[allSprites objectAtIndex :lastRectTouched] currentRect].size.height)
                                        operation:NSCompositeSourceOver];
        [spriteThatsMoving unlockFocus];
        [background lockFocus];
        [[NSColor whiteColor] set];
        NSRectFill(NSMakeRect (0, 0, [background size].width, [background size].height) );
        [[NSColor blackColor] set];
        
        for ( lcv = 0; lcv < [bordersArray count]; lcv++ )
            [[NSBezierPath bezierPathWithRect:
                NSMakeRect([[[bordersArray objectAtIndex :lcv] objectForKey :@"startX"] floatValue], [[[bordersArray objectAtIndex :lcv] objectForKey :@"startY"] floatValue], [[[bordersArray objectAtIndex :lcv] objectForKey :@"differenceX"] intValue], [[[bordersArray objectAtIndex :lcv] objectForKey :@"differenceY"] intValue])]
                stroke];

        for (lcv = 1; lcv < [layers count] + 1; lcv++ )
            for ( lcv2 = 0; lcv2 < [allSprites count]; lcv2++ )
                if ( [[allSprites objectAtIndex: lcv2] layer] == [[[layers objectAtIndex :lcv - 1] objectForKey :@"layer"] intValue] ) // build up each layer
                    [[[allSprites objectAtIndex: lcv2] sprite] compositeToPoint:NSMakePoint([[allSprites objectAtIndex: lcv2] currentRect].origin.x, [[allSprites objectAtIndex: lcv2] currentRect].origin.y) fromRect:NSMakeRect(0,0, [[allSprites objectAtIndex :lcv2] currentRect].size.width, [[allSprites objectAtIndex :lcv2] currentRect].size.height) operation:NSCompositeSourceOver];
                
        [background unlockFocus]; // now I can put the "background" back even though it's really layers
        [self lockFocus];
        [background compositeToPoint:NSMakePoint(0, 0) fromRect:NSMakeRect(0, 0, [background size].width, [background size].height) operation:NSCompositeSourceOver];
        [spriteThatsMoving dissolveToPoint:NSMakePoint([[allSprites objectAtIndex :lastRectTouched] currentRect].origin.x + 3, [[allSprites objectAtIndex :lastRectTouched] currentRect].origin.y + 3) fromRect:NSMakeRect(0, 0, [spriteThatsMoving size].width, [spriteThatsMoving size].height) fraction:.6];
        [self unlockFocus];
        [[self window] flushWindow];
        mouseClickedPos = [self convertPoint: [theEvent locationInWindow] fromView:nil];
        state = movingSprite;
    }
    if ( state == doingCrop ){
        if ( lastRectTouched != -1 ){
            state = performingCrop;
			[background lockFocus];
            [[NSColor whiteColor] set];
            NSRectFill(NSMakeRect (0, 0, [background size].width, [background size].height) );
            [[NSColor blackColor] set];
            for ( lcv = 0; lcv < [bordersArray count]; lcv++ )
                [[NSBezierPath bezierPathWithRect:
                    NSMakeRect([[[bordersArray objectAtIndex :lcv] objectForKey :@"startX"] floatValue], [[[bordersArray objectAtIndex :lcv] objectForKey :@"startY"] floatValue], [[[bordersArray objectAtIndex :lcv] objectForKey :@"differenceX"] intValue], [[[bordersArray objectAtIndex :lcv] objectForKey :@"differenceY"] intValue])]
                    stroke];
            
            for (lcv = 1; lcv < [layers count] + 1; lcv++ )
                for ( lcv2 = 0; lcv2 < [allSprites count]; lcv2++ )
                    if ( [[allSprites objectAtIndex: lcv2] layer] == [[[layers objectAtIndex :lcv - 1] objectForKey :@"layer"] intValue] ) // build up each layer
                        if ( [allSprites objectAtIndex :lastSpriteTouched] != [allSprites objectAtIndex: lcv2] )
                            [[[allSprites objectAtIndex: lcv2] sprite] compositeToPoint:NSMakePoint([[allSprites objectAtIndex: lcv2] currentRect].origin.x, [[allSprites objectAtIndex: lcv2] currentRect].origin.y) fromRect:NSMakeRect(0,0, [[allSprites objectAtIndex :lcv2] currentRect].size.width, [[allSprites objectAtIndex :lcv2] currentRect].size.height) operation:NSCompositeSourceOver];
                        else // it's the one I'm cropping, so make it slightly transparent
							[[[allSprites objectAtIndex: lcv2] sprite] dissolveToPoint:NSMakePoint([[allSprites objectAtIndex :lastRectTouched] currentRect].origin.x + 3, [[allSprites objectAtIndex :lastRectTouched] currentRect].origin.y + 3) fromRect:NSMakeRect(0, 0, [spriteThatsMoving size].width, [spriteThatsMoving size].height) fraction:.6];
			[background unlockFocus]; // now I can put the "background" back even though it's really layers

            mouseClickedPos = [self convertPoint: [theEvent locationInWindow] fromView:nil];
//            NSRect tempRectOne = [[allSprites objectAtIndex :lastRectTouched] currentRect];
//            NSLog ( @"I clicked %f pixels from the left and %f pixels from the bottom", mouseClickedPos.x - tempRectOne.origin.x, mouseClickedPos.y - tempRectOne.origin.y);
        } else
        {
            NSBeep();
            state = normal;
        }
    }        
    if ( state == drawingSquare ){
            [boxC set];
            state = drawingSquare;
            mouseClickedPos = [self convertPoint: [theEvent locationInWindow] fromView:nil];

            [background lockFocus];
            [[NSColor whiteColor] set];
            NSRectFill(NSMakeRect (0, 0, [background size].width, [background size].height) );
            [[NSColor blackColor] set];
            for ( lcv = 0; lcv < [bordersArray count]; lcv++ )
                [[NSBezierPath bezierPathWithRect:
                    NSMakeRect([[[bordersArray objectAtIndex :lcv] objectForKey :@"startX"] floatValue], [[[bordersArray objectAtIndex :lcv] objectForKey :@"startY"] floatValue], [[[bordersArray objectAtIndex :lcv] objectForKey :@"differenceX"] intValue], [[[bordersArray objectAtIndex :lcv] objectForKey :@"differenceY"] intValue])]
                    stroke];
            
            for (lcv = 1; lcv < [layers count] + 1; lcv++ )
                for ( lcv2 = 0; lcv2 < [allSprites count]; lcv2++ )
                    if ( [[allSprites objectAtIndex: lcv2] layer] == [[[layers objectAtIndex :lcv - 1] objectForKey :@"layer"] intValue] ) // build up each layer
                        [[[allSprites objectAtIndex: lcv2] sprite] compositeToPoint:NSMakePoint([[allSprites objectAtIndex: lcv2] currentRect].origin.x, [[allSprites objectAtIndex: lcv2] currentRect].origin.y) fromRect:NSMakeRect(0,0, [[allSprites objectAtIndex :lcv2] currentRect].size.width, [[allSprites objectAtIndex :lcv2] currentRect].size.height) operation:NSCompositeSourceOver];
            [background unlockFocus]; // now I can put the "background" back even though it's really layers            
    }
    if ( state == drawingBubble || state == drawingThoughtBubble){
        
    if ( state == drawingBubble )
        [bubbleC set];
    if ( state == drawingThoughtBubble)
        [thoughBubbleC set];
    
mouseClickedPos = [self convertPoint: [theEvent locationInWindow] fromView:nil];
    [background lockFocus];
        [[NSColor whiteColor] set];
        NSRectFill(NSMakeRect (0, 0, [background size].width, [background size].height) );
        [[NSColor blackColor] set];
        for ( lcv = 0; lcv < [bordersArray count]; lcv++ )
            [[NSBezierPath bezierPathWithRect:
                NSMakeRect([[[bordersArray objectAtIndex :lcv] objectForKey :@"startX"] floatValue], [[[bordersArray objectAtIndex :lcv] objectForKey :@"startY"] floatValue], [[[bordersArray objectAtIndex :lcv] objectForKey :@"differenceX"] intValue], [[[bordersArray objectAtIndex :lcv] objectForKey :@"differenceY"] intValue])]
                stroke];
        
        for (lcv = 1; lcv < [layers count] + 1; lcv++ )
            for ( lcv2 = 0; lcv2 < [allSprites count]; lcv2++ )
                if ( [[allSprites objectAtIndex: lcv2] layer] == [[[layers objectAtIndex :lcv - 1] objectForKey :@"layer"] intValue] ) // build up each layer
                    [[[allSprites objectAtIndex: lcv2] sprite] compositeToPoint:NSMakePoint([[allSprites objectAtIndex: lcv2] currentRect].origin.x, [[allSprites objectAtIndex: lcv2] currentRect].origin.y) fromRect:NSMakeRect(0,0, [[allSprites objectAtIndex :lcv2] currentRect].size.width, [[allSprites objectAtIndex :lcv2] currentRect].size.height) operation:NSCompositeSourceOver];
    [background unlockFocus]; // now I can put the "background" back even though it's really layers 
    }
    if ( state == makingTail1 )
        mouseClickedPos = [self convertPoint: [theEvent locationInWindow] fromView:nil]; // just grab where they clicked	
}

- (void)mouseDragged:(NSEvent *)event
{

    NSPoint mousePoint = [self convertPoint: [event locationInWindow] fromView:nil];
    int differenceX = mousePoint.x - mouseClickedPos.x;
    int differenceY = mousePoint.y - mouseClickedPos.y;
    // !!!!! delete from array if dragged off screen
    if ( state == movingSprite ){
        [self lockFocus];

        [background compositeToPoint: NSMakePoint(0,0) fromRect:NSMakeRect(0,0, [background size].width, [background size].height) operation:NSCompositeSourceOver];
        [spriteThatsMoving dissolveToPoint:NSMakePoint([[allSprites objectAtIndex :lastRectTouched] currentRect].origin.x + differenceX, [[allSprites objectAtIndex :lastRectTouched] currentRect].origin.y + differenceY) fromRect:NSMakeRect(0, 0, [spriteThatsMoving size].width, [spriteThatsMoving size].height) fraction:.6];
        [self unlockFocus];
        [[self window] flushWindow];
    }
    if ( state == drawingSquare || state == performingCrop){
        [self lockFocus];
        [background compositeToPoint: NSMakePoint(0,0) fromRect:NSMakeRect(0,0, [background size].width, [background size].height) operation:NSCompositeSourceOver];
        [[NSColor blackColor] set];
        [[NSBezierPath bezierPathWithRect: NSMakeRect(mouseClickedPos.x, mouseClickedPos.y, differenceX, differenceY)] stroke];
        [self unlockFocus];
        [[self window] flushWindow];

    }
    if ( state == drawingBubble || state == drawingThoughtBubble){
        [self lockFocus];
        [background compositeToPoint: NSMakePoint(0,0) fromRect:NSMakeRect(0,0, [background size].width, [background size].height) operation:NSCompositeSourceOver];
        [[NSColor colorWithCalibratedWhite: 1 alpha: .5] set];
        [[NSBezierPath bezierPathWithOvalInRect:NSMakeRect( mouseClickedPos.x, mouseClickedPos.y, differenceX, differenceY)] fill];
        [[NSColor blackColor] set];
        [[NSBezierPath bezierPathWithOvalInRect:NSMakeRect( mouseClickedPos.x, mouseClickedPos.y, differenceX, differenceY)] stroke];
        [self unlockFocus];
        [[self window] flushWindow];
        
    }
}

- (void)mouseUp:(NSEvent *)theEvent
{
    NSPoint mousePoint = [self convertPoint: [theEvent locationInWindow] fromView:nil];
    int differenceX = mousePoint.x - mouseClickedPos.x;
    int differenceY = mousePoint.y - mouseClickedPos.y;

    if ( state == movingSprite ){
        [[NSCursor arrowCursor] set];
        [[allSprites objectAtIndex :lastRectTouched] setRect :NSMakeRect( [[allSprites objectAtIndex :lastRectTouched] currentRect].origin.x + differenceX, [[allSprites objectAtIndex :lastRectTouched] currentRect].origin.y + differenceY, [spriteThatsMoving size].width, [spriteThatsMoving size].height) ];
// I don't know why, but without this, I get part of the sprite I'm moving in the next bubble or text I move
        [spriteThatsMoving lockFocus];
        [[NSColor clearColor] set];
        NSRectFill(NSMakeRect(0,0, [spriteThatsMoving size].width, [spriteThatsMoving size].height));
        [spriteThatsMoving unlockFocus];
//		lastRectTouched = -1;
        state = normal;
		[self setNeedsDisplay :YES];
		return;
    }
    
    if ( state == drawingSquare ){
        [[NSCursor arrowCursor] set];
        NSMutableDictionary *border = [NSMutableDictionary dictionaryWithCapacity: 1];

        [border setObject:[NSNumber numberWithDouble :mouseClickedPos.x] forKey:@"startX"];
        [border setObject:[NSNumber numberWithDouble :mouseClickedPos.y] forKey:@"startY"];
        [border setObject:[NSNumber numberWithInt :differenceX] forKey:@"differenceX"];
        [border setObject:[NSNumber numberWithInt :differenceY] forKey:@"differenceY"];
        [bordersArray addObject: border];
    }
    
    if ( state == performingCrop ){
        int lowestX = mousePoint.x < mouseClickedPos.x ? mousePoint.x : mouseClickedPos.x;
        int lowestY = mousePoint.y < mouseClickedPos.y ? mousePoint.y : mouseClickedPos.y;

        NSRect tempRectOne = [[allSprites objectAtIndex :lastRectTouched] currentRect];
        [[allSprites objectAtIndex :lastSpriteTouched] setRect: NSMakeRect (lowestX, lowestY, abs(differenceX), abs(differenceY))]; // this changes the rect so it displays properly on the next update
        [[allSprites objectAtIndex :lastSpriteTouched] crop :abs(lowestX - tempRectOne.origin.x) :abs(lowestY - tempRectOne.origin.y) :abs(differenceX) :abs(differenceY)]; // this actually changes the size of the image to be the new size
        state = normal;
        lastRectTouched = -1;
    }
    
    if ( state == drawingBubble || state == drawingThoughtBubble){
        if ( abs(differenceX) > 10 && abs(differenceY) > 10 ){ // may as well not make a small rect if they click by accident
            NSBezierPath *bubbleOutline = [NSBezierPath bezierPathWithOvalInRect:NSMakeRect( 1, 1, abs(differenceX) - 2, abs(differenceY) - 2)];
            bubble = [[NSImage alloc] initWithSize :NSMakeSize (abs(differenceX), abs(differenceY))];
            bubbleLowestX = mousePoint.x < mouseClickedPos.x ? mousePoint.x : mouseClickedPos.x;
            bubbleLowestY = mousePoint.y < mouseClickedPos.y ? mousePoint.y : mouseClickedPos.y;
            // I need these lowest vars so I can trick it into thinking I'm clicking somewhere near 0,0, where my image thinks
            // it is.
            [bubbleOutline setLineWidth :2];
            [bubble lockFocus];
            [[NSColor clearColor] set];
            NSRectFill(NSMakeRect(0,0, abs(differenceX), abs(differenceY)));
            [[NSColor whiteColor] set];
            [bubbleOutline fill];
            [[NSColor blackColor] set];
            [bubbleOutline stroke];
            [bubble unlockFocus];
            [self lockFocus];
            [[NSBezierPath bezierPathWithOvalInRect:NSMakeRect( mouseClickedPos.x, mouseClickedPos.y, differenceX, differenceY)] stroke];
            [self unlockFocus];
            [[self window] flushWindow];
            if ( state == drawingBubble )
                state = makingTail1;
            if ( state == drawingThoughtBubble ){
                state = drawingTinyBubbles;
                bubbleCount = 3;
                [self addCharacter: bubble :[[[layers objectAtIndex :layerWorkingOn] objectForKey:@"layer"] intValue] :bubbleLowestX :bubbleLowestY];
                [bubble release];

            }
            return; // don't redraw just yet, my pretty!
        }
    }
    if ( state == drawingDialog ){
        NSMutableDictionary *tempDictionary;
		NSMutableDictionary *dialogTrackerItem;
        mousePoint = [self convertPoint: [theEvent locationInWindow] fromView:nil];        
        tempDictionary = [NSMutableDictionary dictionary];
		dialogTrackerItem = [NSMutableDictionary dictionary];
        [tempDictionary setObject:fontToRender forKey:NSFontAttributeName];
        [tempDictionary setObject:[NSColor blackColor] forKey:NSForegroundColorAttributeName];
		[dialogTrackerItem setObject:[NSString stringWithString:dialogToRender] forKey:@"dialog"];
		[dialogTrackerItem setObject:[NSString stringWithFormat:@"%d", [allSprites count]] forKey:@"spriteNumber"];
		[dialogTracking addObject: dialogTrackerItem];
		
        bubble = [[NSImage alloc] initWithSize :NSMakeSize([dialogToRender sizeWithAttributes :tempDictionary].width + 3, [dialogToRender sizeWithAttributes :tempDictionary].height + 2)];
// I am such a fool for not knowing about sizeWithAttributes earlier! What a fool!
// oh, and the + X is just to give a bit of breathing space, that's all.
        // may as well reuse bubble here, I have it hanging around, after all

        [bubble lockFocus];
        [[NSColor clearColor] set];
        NSRectFill(NSMakeRect(0,0, [bubble size].width, [bubble size].height));
//            NSLog (@"%f, %f", [dialogToRender sizeWithAttributes :tempDictionary].width, [dialogToRender sizeWithAttributes :tempDictionary].height);
        [dialogToRender drawAtPoint:NSMakePoint(0, 0) withAttributes:tempDictionary];
        [bubble unlockFocus];
        [self addCharacter: bubble :[[[layers objectAtIndex :layerWorkingOn] objectForKey:@"layer"] intValue] :mousePoint.x :mousePoint.y];
        
        [bubble release];
        state = normal;
        [[NSCursor arrowCursor] set];
		[self setNeedsDisplay :YES];
        return;
    }
    if ( state == drawingTinyBubbles ){
        NSBezierPath *bubbleOutline = [NSBezierPath bezierPathWithOvalInRect:NSMakeRect( 1, 1, bubbleCount * 3, bubbleCount * 3)];
        bubble = [[NSImage alloc] initWithSize :NSMakeSize (bubbleCount * 4, bubbleCount * 4)];
        bubbleLowestX = mousePoint.x;
        bubbleLowestY = mousePoint.y;
        // I need these lowest vars so I can trick it into thinking I'm clicking somewhere near 0,0, where my image thinks
        // it is.
        [bubbleOutline setLineWidth :2];
        [bubble lockFocus];
        [[NSColor clearColor] set];
        NSRectFill(NSMakeRect(0,0, bubbleCount * 4, bubbleCount * 4));
        [[NSColor whiteColor] set];
        [bubbleOutline fill];
        [[NSColor blackColor] set];
        [bubbleOutline stroke];
        [bubble unlockFocus];
        bubbleCount--;
        if (bubbleCount == 0)
            state = drawingThoughtBubble;
        [self addCharacter: bubble :[[[layers objectAtIndex :layerWorkingOn] objectForKey:@"layer"] intValue] :bubbleLowestX :bubbleLowestY];
        [bubble release];
        [self setNeedsDisplay: YES];
        return;
    }
    if ( state == makingTail1 ){
        [tailOutline removeAllPoints]; // give me an empty one
        // first I have to adjust the size if I've clicked outside the current boarder of the sprite.
        if ( bubbleLowestX + [bubble size].width < mousePoint.x ){
            [bubble setSize :NSMakeSize( abs( [bubble size].width + bubbleLowestX - mousePoint.x) + [bubble size].width, [bubble size].height)];
        }
        if ( bubbleLowestY + [bubble size].height <  mousePoint.y ){
            [bubble setSize :NSMakeSize([bubble size].width, abs( [bubble size].height + bubbleLowestY - mousePoint.y) + [bubble size].height)];
        }
        // going negatively is a bit more tricky, I'm afraid.
        if ( bubbleLowestX > mousePoint.x ){
            NSImage *imageShift = [[NSImage alloc] initWithSize :[bubble size]];
            // I have to do this to shift the current bubble over. As I'm adding space in the - direction, if I don't shift what
            // I have in there now, it'll just put the tail right over the bubble
            [imageShift lockFocus];
            [bubble compositeToPoint:NSMakePoint(0, 0)
                            fromRect:NSMakeRect(0, 0, [bubble size].width, [bubble size].height)
                           operation:NSCompositeSourceOver];
            [imageShift unlockFocus];
            [bubble setSize :NSMakeSize( bubbleLowestX -  mousePoint.x + [bubble size].width, [bubble size].height)];
            [bubble lockFocus];
            [[NSColor clearColor] set];
            NSRectFill( NSMakeRect (0, 0, [bubble size].width, [bubble size].height));
            [imageShift compositeToPoint:NSMakePoint(bubbleLowestX -  mousePoint.x, 0)
                                fromRect:NSMakeRect(0, 0, [bubble size].width, [bubble size].height)
                               operation:NSCompositeSourceOver];
            [bubble unlockFocus];
            bubbleLowestX = mousePoint.x;
        }
        if ( bubbleLowestY > mousePoint.y ){
            NSImage *imageShift = [[NSImage alloc] initWithSize :[bubble size]];
            // I have to do this to shift the current bubble over. As I'm adding space in the - direction, if I don't shift what
            // I have in there now, it'll just put the tail right over the bubble
            [imageShift lockFocus];
            [bubble compositeToPoint:NSMakePoint(0, 0)
                            fromRect:NSMakeRect(0, 0, [bubble size].width, [bubble size].height)
                           operation:NSCompositeSourceOver];
            [imageShift unlockFocus];
            [bubble setSize :NSMakeSize( [bubble size].width, bubbleLowestY -  mousePoint.y + [bubble size].height)];
            [bubble lockFocus];
            [[NSColor clearColor] set];
            NSRectFill( NSMakeRect (0, 0, [bubble size].width, [bubble size].height));
            [imageShift compositeToPoint:NSMakePoint(0, bubbleLowestY -  mousePoint.y)
                                fromRect:NSMakeRect(0, 0, [bubble size].width, [bubble size].height)
                               operation:NSCompositeSourceOver];
            [bubble unlockFocus];
            bubbleLowestY = mousePoint.y;
        }
        // these abs( ) guys compensate for the fact that though I am making a bubble at some arbitrary x,y, I'm making an NSImage
        // starting at 0,0 with a width and height.
        [tailOutline moveToPoint : NSMakePoint ( abs(bubbleLowestX - mousePoint.x), abs(bubbleLowestY - mousePoint.y) )];
        trianglePoint = NSMakePoint ( abs(bubbleLowestX - mousePoint.x), abs(bubbleLowestY - mousePoint.y) );
        
        originalTailPoint = mousePoint;
        [self lockFocus];
        [NSBezierPath strokeLineFromPoint:NSMakePoint( mousePoint.x, mousePoint.y) toPoint:NSMakePoint (mouseClickedPos.x, mouseClickedPos.y)];
        [self unlockFocus];
        [[self window] flushWindow];

        state = makingTail2;
        return;
    }
    if ( state == makingTail2 ){
        [tailOutline lineToPoint: NSMakePoint ( abs(bubbleLowestX - mousePoint.x), abs(bubbleLowestY - mousePoint.y ))];
        lineOnePoint = NSMakePoint ( abs(bubbleLowestX - mousePoint.x), abs(bubbleLowestY - mousePoint.y ));
        
        [self lockFocus];
        [NSBezierPath strokeLineFromPoint:NSMakePoint( mousePoint.x, mousePoint.y) toPoint:NSMakePoint (mouseClickedPos.x, mouseClickedPos.y)];
        [self unlockFocus];
        
        [[self window] flushWindow];
        state = makingTail3;
        return;
    }
    if ( state == makingTail3 ){
        
        [tailOutline lineToPoint: NSMakePoint ( abs(bubbleLowestX - mousePoint.x), abs(bubbleLowestY - mousePoint.y ))];
        lineTwoPoint = NSMakePoint ( abs(bubbleLowestX - mousePoint.x), abs(bubbleLowestY - mousePoint.y ));

        [tailOutline lineToPoint: NSMakePoint ( abs(bubbleLowestX - originalTailPoint.x), abs(bubbleLowestY - originalTailPoint.y))];
        
        [self lockFocus];
        [NSBezierPath strokeLineFromPoint:NSMakePoint( mousePoint.x, mousePoint.y) toPoint:NSMakePoint (mouseClickedPos.x, mouseClickedPos.y)];
        [self unlockFocus];
        [[self window] flushWindow];
        [tailOutline setLineWidth :2];
        [bubble lockFocus];

        [[NSColor whiteColor] set];
        [tailOutline fill];     
        [[NSColor blackColor] set];
        [NSBezierPath strokeLineFromPoint:trianglePoint toPoint:lineOnePoint];
        [NSBezierPath strokeLineFromPoint:trianglePoint toPoint:lineTwoPoint];

        [bubble unlockFocus];
        state = drawingBubble;
        [self addCharacter: bubble :[[[layers objectAtIndex :layerWorkingOn] objectForKey:@"layer"] intValue] :bubbleLowestX :bubbleLowestY];
        [bubble release];
    }
	// If I get all the way down here I'm just clicking on nothing to get rid of the selection of the last thing I clicked on
	//state = normal;
   if( lastRectTouched = -1 )
	lastSpriteTouched = -1;
    [self setNeedsDisplay :YES];
}

- (void)pasteSprite:(int)visableX :(int)visableY{
    if ( spriteClipboard != nil ){
        [self addCharacter: [spriteClipboard sprite] :[[[layers objectAtIndex :layerWorkingOn] objectForKey:@"layer"] intValue] :visableX :visableY];
//        [spriteClipboard release];
//        spriteClipboard = nil; // though the release should do it, I seem to still have it available
        lastSpriteTouched = -1;
    }
}

- (void)replaceSelectedTextWithThis :(NSString *)dialog :(NSFont *)fontToRenderWith
{
	int lcv;

	[dialogToRender setString :dialog];
    fontToRender = fontToRenderWith;

	for ( lcv = 0; lcv < [dialogTracking count]; lcv++ ){ // I need to reduce the spriteNumber of anything after the one I've just deleted
		if ( [[[dialogTracking objectAtIndex: lcv] objectForKey:@"spriteNumber"] intValue] == lastSpriteTouched ){
			[[dialogTracking objectAtIndex: lcv] setValue:dialog forKey:@"dialog"];
		}
	}
	[self deleteSprite];
}

- (void)renderText :(NSString *)dialog :(NSFont *)fontToRenderWith
{
//    NSLog (@"%f", [fontToRenderWith pointSize]);
    [dialogToRender setString :dialog];
    fontToRender = fontToRenderWith;
}

- (void)removeBoarders
{
if ( [bordersArray count] != 0 ){
        [bordersArray removeObjectAtIndex: [bordersArray count] - 1];
        [self setNeedsDisplay :YES];
    }

}

- (void)rotateCurrentSprite :(float)rotateFactor
{
    [[NSAffineTransform transform] set];        
    if ( lastSpriteTouched != -1 ){
        int differenceX = 0, differenceY = 0;
        NSImage *tempImage = [[NSImage alloc] initWithSize: [[[allSprites objectAtIndex :lastSpriteTouched] sprite] size]];
        NSAffineTransform* transform = [NSAffineTransform transform];

        if ( [tempImage size].width > [tempImage size].height ){
            // it's not square, so it'll have to be rotated and the image resized so nothing gets cut off
            // it's clumsy, but for now just make it a square. To be really slick, I would resize it to JUST fit into the square
            differenceY = [tempImage size].width - [tempImage size].height;
            [[[allSprites objectAtIndex :lastSpriteTouched] sprite] setScalesWhenResized :NO];
            [[[allSprites objectAtIndex :lastSpriteTouched] sprite] setSize :NSMakeSize ([tempImage size].width,
                                                                                         [tempImage size].width)];
            [tempImage setSize :NSMakeSize ([tempImage size].width,
                                            [tempImage size].width)];
            
            [[allSprites objectAtIndex :lastSpriteTouched] setRect : NSMakeRect( [[allSprites objectAtIndex :lastSpriteTouched] currentRect].origin.x, [[allSprites objectAtIndex :lastSpriteTouched] currentRect].origin.y,
                                                                                 [tempImage size].width, [tempImage size].height )];
        }
        if ( [tempImage size].width < [tempImage size].height ){
            // it's not square, so it'll have to be rotated and the image resized so nothing gets cut off
            // it's clumsy, but for now just make it a square. To be really slick, I would resize it to JUST fit into the square
            differenceX = [tempImage size].height - [tempImage size].width;
            [[[allSprites objectAtIndex :lastSpriteTouched] sprite] setScalesWhenResized :NO];
            [[[allSprites objectAtIndex :lastSpriteTouched] sprite] setSize :NSMakeSize ([tempImage size].height,
                                                                                         [tempImage size].height)];
            [tempImage setSize :NSMakeSize ([tempImage size].height,
                                            [tempImage size].height)];
            [[allSprites objectAtIndex :lastSpriteTouched] setRect : NSMakeRect( [[allSprites objectAtIndex :lastSpriteTouched] currentRect].origin.x, [[allSprites objectAtIndex :lastSpriteTouched] currentRect].origin.y,
                                                                                 [[allSprites objectAtIndex :lastSpriteTouched] currentRect].size.height, [[allSprites objectAtIndex :lastSpriteTouched] currentRect].size.height )];
        }
        // trouble is, now the sprite isn't in the middle anymore!
        [tempImage lockFocus];
        [[[allSprites objectAtIndex :lastSpriteTouched] sprite] drawAtPoint:NSMakePoint(differenceX / 2, differenceY / 2) fromRect:NSMakeRect(0,0, [[[allSprites objectAtIndex :lastSpriteTouched] sprite] size].width, [[[allSprites objectAtIndex :lastSpriteTouched] sprite] size].height ) operation:NSCompositeSourceOver fraction:1];
        [tempImage unlockFocus];
        [[[allSprites objectAtIndex :lastSpriteTouched] sprite ] lockFocus];
        // now, clear out the old, and put the temp in it's place
        [[NSColor clearColor] set];
        NSRectFill (NSMakeRect(0,0, [tempImage size].width, [tempImage size].height ));
        [tempImage drawAtPoint :NSMakePoint(0, 0) fromRect:NSMakeRect(0,0, [tempImage size].width, [tempImage size].height ) operation:NSCompositeSourceOver fraction:1];
        [[[allSprites objectAtIndex :lastSpriteTouched] sprite] unlockFocus];
// somehow, right here, after scaling > 1.96 you get some kind of white bar in your sprite. I have no idea why.
        
// now that it's in the middle again, we can procede with our rotation. Seesh, wish there was an easier way to do this!
        [transform translateXBy:[tempImage size].width / 2 yBy:[tempImage size].height / 2];
        [transform rotateByDegrees:rotateFactor];
        [transform translateXBy:-[tempImage size].width / 2 yBy:-[tempImage size].height / 2];
        
        [tempImage lockFocus];
        [[NSColor clearColor] set];
        NSRectFill (NSMakeRect(0,0, [tempImage size].width, [tempImage size].height ));
        [transform concat];
        [[[allSprites objectAtIndex :lastSpriteTouched] sprite] drawAtPoint:NSMakePoint(0, 0) fromRect:NSMakeRect(0, 0,[tempImage size].width, [tempImage size].height ) operation:NSCompositeSourceOver fraction:1];
        [tempImage unlockFocus];

        [[NSAffineTransform transform] set];        
        [[[allSprites objectAtIndex :lastSpriteTouched] sprite ] lockFocus];
        // now, clear out the old, and put the temp in it's place        
        [[NSColor clearColor] set];
        NSRectFill (NSMakeRect(0,0, [[[allSprites objectAtIndex :lastSpriteTouched] sprite] size].width, [[[allSprites objectAtIndex :lastSpriteTouched] sprite] size].height ));
        [tempImage drawAtPoint :NSMakePoint(0, 0) fromRect:NSMakeRect(0,0, [tempImage size].width, [tempImage size].height ) operation:NSCompositeSourceOver fraction:1];
        [[[allSprites objectAtIndex :lastSpriteTouched] sprite] unlockFocus];
        [self setNeedsDisplay:YES];
    }
}

- (void)dealloc
{
    [background release];
    [spriteThatsMoving release];
    [bubble release];
    [spriteClipboard release];
    [allSprites removeAllObjects];
    [trackingRects removeAllObjects];
    [layers removeAllObjects];
    [tailOutline release];
    [handC release];
    [boxC release];
    [bubbleC release];
    [thoughBubbleC release];
	[dialogTracking release];
	
	[super dealloc];
    // I wash my hands of you freaks! Begone!
}

- (void)scaleCurrentSprite :(float)scaleFactor
{
    if ( lastSpriteTouched != -1 ){
        [[[allSprites objectAtIndex :lastSpriteTouched] sprite] setScalesWhenResized: YES];
        [[[allSprites objectAtIndex :lastSpriteTouched] sprite] setSize :NSMakeSize ([[[allSprites objectAtIndex :lastSpriteTouched] sprite] size].width * scaleFactor,
                                                                           [[[allSprites objectAtIndex :lastSpriteTouched] sprite] size].height * scaleFactor)];
        [[allSprites objectAtIndex :lastSpriteTouched] setRect : NSMakeRect( [[allSprites objectAtIndex :lastSpriteTouched] currentRect].origin.x, [[allSprites objectAtIndex :lastSpriteTouched] currentRect].origin.y,
                                                                             [[allSprites objectAtIndex :lastSpriteTouched] currentRect].size.width * scaleFactor, [[allSprites objectAtIndex :lastSpriteTouched] currentRect].size.height * scaleFactor )];
        [self setNeedsDisplay: YES];
    }
}

- (void)makeSpriteTransparent: (int)transparencyValue
{
    if ( lastSpriteTouched != -1 ){
        [[allSprites objectAtIndex :lastSpriteTouched] setTransparency: transparencyValue];
        [self setNeedsDisplay: YES];
    }
}

- (void)saveImage :(NSString *)saveLocation
{
    NSString *type;
    NSBitmapImageRep* imageRep;
    NSBitmapImageFileType typeCode = -99;
    int lcv, lcv2, borderLCV;
    double compression = 1.0;

    type = [[saveLocation pathExtension] lowercaseString]; // get a dictionary of values from the file manager about this file and get the type from them

    typeCode = NSTIFFFileType; // I could check if type is empty here, but I'm lazy, so if you don't tell
    // me, you get a tiff.
    
    if ( [type isEqualTo: @"jpg"])
        typeCode = NSJPEGFileType;
    if ( [type isEqualTo: @"png"] )
        typeCode = NSPNGFileType;
    if ( [type isEqualTo: @"tiff"]  || [type isEqualTo: @"tif"] )
        typeCode = NSTIFFFileType;
    
    [background lockFocus];
    [[NSColor whiteColor] set];
    NSRectFill(NSMakeRect(0,0, [background size].width, [background size].height));
    for (lcv = 1; lcv < [layers count] + 1; lcv++ ){
        //        NSLog (@"Now checking layer %@", [[layers objectAtIndex :lcv - 1] objectForKey :@"layer"] );
        if ( lcv == 2 ){
            [[NSColor blackColor] set];
            for ( borderLCV = 0; borderLCV < [bordersArray count]; borderLCV++ )
                [[NSBezierPath bezierPathWithRect:
                    NSMakeRect([[[bordersArray objectAtIndex :borderLCV] objectForKey :@"startX"] floatValue], [[[bordersArray objectAtIndex :borderLCV] objectForKey :@"startY"] floatValue], [[[bordersArray objectAtIndex :borderLCV] objectForKey :@"differenceX"] intValue], [[[bordersArray objectAtIndex :borderLCV] objectForKey :@"differenceY"] intValue])]
                    stroke];
        }
        
        for ( lcv2 = 0; lcv2 < [allSprites count]; lcv2++ ){
            //           NSLog (@"THis spirte is in layer %d", [[allSprites objectAtIndex: lcv2] layer] );            
            if ( [[allSprites objectAtIndex: lcv2] layer] == [[[layers objectAtIndex :lcv - 1] objectForKey :@"layer"] intValue] ){ // build up each layer

                [[[allSprites objectAtIndex: lcv2] sprite]
                        dissolveToPoint:NSMakePoint([[allSprites objectAtIndex: lcv2] currentRect].origin.x, [[allSprites objectAtIndex: lcv2] currentRect].origin.y)
                        fromRect:NSMakeRect(0, 0, [[[allSprites objectAtIndex: lcv2] sprite] size].width, [[[allSprites objectAtIndex: lcv2] sprite] size].height)
                        fraction:[[allSprites objectAtIndex: lcv2] transparency] * .01];
            }
        }
    }
        
    imageRep = [[NSBitmapImageRep alloc] initWithFocusedViewRect:NSMakeRect(0, 0, [background size].width, [background size].height)];

    [background unlockFocus]; // now I can put the "background" back even though it's really layers

    [[imageRep representationUsingType:typeCode properties:[NSDictionary dictionaryWithObject:[NSNumber numberWithFloat:compression] forKey:NSImageCompressionFactor]] writeToFile:saveLocation atomically:YES];
}

- (void)toolChange :(int)newState
{
    state = newState; // because that's really all I'm doing....
    if ( state == drawingSquare )
        [boxC set];
    if ( state == drawingBubble || state == drawingDialog )
        [bubbleC set];
    if ( state == drawingThoughtBubble )
        [thoughBubbleC set];
    if ( state == doingCrop )
        [cropC set];
}

- (NSString *)textToEdit
{
	int lcv;
	for ( lcv = 0; lcv < [dialogTracking count]; lcv++ ){
		if ( [[[dialogTracking objectAtIndex: lcv] objectForKey:@"spriteNumber"] intValue] == lastSpriteTouched ){
			return [[dialogTracking objectAtIndex: lcv] objectForKey:@"dialog"];
		}
	}
	return @"";
}

- (NSMutableArray *)spritesForSaving
{
    return allSprites;
}

- (void)nudge:(int)hor :(int)vert
{
    if ( lastSpriteTouched != -1 ){

        [[allSprites objectAtIndex :lastSpriteTouched] setRect :NSMakeRect( [[allSprites objectAtIndex :lastSpriteTouched] currentRect].origin.x + hor, [[allSprites objectAtIndex :lastSpriteTouched] currentRect].origin.y + vert, [spriteThatsMoving size].width, [spriteThatsMoving size].height) ];
        [self setNeedsDisplay :YES];
    } else
        NSBeep();
}

@end
